#ifndef __INC_HTTPConnection
#define __INC_HTTPConnection

#include "MKCString.h"
#include "INetSocket.h"
#include "HTTPCookies.h"

#define OUTPUT_BUFFER_SIZE		1024
#define INPUT_BUFFER_SIZE		2048
#define HTTP_DEFAULT_TIMEOUT	1000
#define HTTP_PORT				80
#define HTTPS_PORT 				443

#define errSocketNotSet 			1000
#define errSocketAlreadyConnected 	1001
#define errSocketNoConnected 		1002
#define errSocketError 				1003

class HTTPRequest;
class HTTPResponse;

class HTTPConnection : public HTTPBase
{
public:
	HTTPConnection(unsigned short inBufferSize = INPUT_BUFFER_SIZE, unsigned short outBufferSize = OUTPUT_BUFFER_SIZE);
	virtual ~HTTPConnection();

	HTTPRequest*  	getRequest() { return m_request; }
	HTTPResponse* 	getResponse() { return m_response; }
	short			getTimeOut() { return m_timeOut; }
	HTTPCookies*	getCookies() { return &m_cookies; }

	void setRequest(HTTPRequest* req) { m_request =  req; }
	void setResponse(HTTPResponse *resp) { m_response = resp; }
	void setTimeOut(unsigned short timeOut);
	
	bool parseURL(char* url, CMKString* domainName, short *port, CMKString* path, CMKString* fileName, CMKString* queryString);
		
	void setEndPoint(char* domain, unsigned short port);
	//bool connect(); 
	bool connect(char *domain, unsigned short port);
	bool connected();
	void disconnect();

	short executeRequest(CMKString* pLastStringSend = NULL); // if pass pointer CMKString => return the last string send to server
	
	unsigned short getPort() { return m_port; }
	INetSocket*	GetSocket() { return m_socket; }
	
protected:
	INetSocket*	m_socket;
	
private: 	
	short 	loadResponse();
	short 	loadHeader();
	short 	loadContent();
	short 	fillInputBuffer(long maxSize = -1);

	void buildOutputBuffer();
	void addCommand();
	void addHeader(char* name, char* value);
	void addHeaders();
	void addCookies();
	void addContent();
	void clearBuffers();
	
	bool doConnect();
	bool doDisconnect();

	virtual unsigned short GetDefaultPort() const { return HTTP_PORT; }

private:
	short 		 	m_timeOut;
	bool		 	m_secure;
	CMKString 	 	m_domain;
	unsigned short 	m_port;
	HTTPRequest*	m_request;
	HTTPResponse*	m_response;
	HTTPCookies  	m_cookies;
	unsigned short	m_inputBufferSize;
	char*			m_inputBuffer;
	char*			m_inputBufferPt;
	unsigned short 	m_ibLen;
	char* 			m_outputBuffer;
	char*			buffer;
};
#endif
