#ifndef __INC_CMGLinkedList
#define __INC_CMGLinkedList


#define NULL 0

template <class DType> 
class CMGListNode {             // inner class CMGListNode
public:
		DType data;          // Data element of list node
		CMGListNode* next;          // Pointer to next list node
		CMGListNode* prev;
		CMGListNode(DType e, CMGListNode* n = NULL, CMGListNode* p = NULL) : data(e), next(n), prev(p) {}
		CMGListNode(CMGListNode* n = NULL, CMGListNode* p = NULL) : next(n), prev(p) {}
};

template <class DType> 
class CMGLinkedList {
public:
	CMGLinkedList(bool releaseData = false);          // Constructor
	~CMGLinkedList();          // Destructor

	DType	current();
	bool	eof();
	bool	bof();

	void	insert( DType, bool atEnd = false );		// Insert element at given position

	int find_elem(  DType& e ); 	// Find list element with value e
									// set curr before found element
									// return -1 if e is not found


	DType remove();				// Remove link after curr, return its value
									// curr and curr->next are not NULL!
	
	DType remove_this();			// Remove link curr, return its value
									// curr is not NULL and not head element!

	int  length() const;			// Return current length of List
	bool isEmpty() const;			// Return TRUE if List is empty

	void toHead();					// set curr to head of list
	void toTail();					// set curr to tail of list

	void next();					// set curr to curr->next
	void previous();				// set curr to curr->prev
	void removeAll();

	bool m_releaseData;

private:
	CMGListNode<DType>* head;		// Pointer to list header
	CMGListNode<DType>* tail;		// Pointer to last elem in list
	CMGListNode<DType>* curr;		// Pointer to current list node
	int   len;						// current length of list
};


template <class DType> CMGLinkedList<DType>::CMGLinkedList(bool releaseData )                    // Constructor
{
  m_releaseData = releaseData;
  tail = curr = head = NULL;
  len = 0;
}

template <class DType> CMGLinkedList<DType>::~CMGLinkedList()
{ 
	removeAll();
}

template <class DType> void CMGLinkedList<DType>::removeAll() {
    CMGListNode<DType> *aux;
	curr = head;
	while (curr) {
  		aux = curr->next;
  		if ((m_releaseData) && (curr->data))
  			delete (curr->data);
  		delete curr;
  		curr = aux;
	}
	head = NULL;
	tail = NULL;
}

template <class DType> void CMGLinkedList<DType>::insert(DType item, bool atEnd ) {	// Insert DType after position curr
	if (curr == NULL || atEnd) {
		if (head == NULL)
			curr = tail = head = new CMGListNode<DType>(item);
		else {
			curr = new CMGListNode<DType>(item, head, NULL);
			curr->next->prev = curr;
			head = curr;
		}
	}
	else{
		curr->next = new CMGListNode<DType>(item, curr->next, curr);
		if( tail == curr )
			tail = curr->next;
		else
			curr->next->next->prev = curr->next;
		curr = curr->next;
	}
	len++;
}


template <class DType> int CMGLinkedList<DType>::find_elem(DType& item )
{
	// Find item in CMGLinkedList, set curr before found element
	// return 1 if item is found, else -1

	if (head == NULL)
		return -1;

	CMGListNode<DType>* tmp = head;
	while (( tmp != NULL ) && !(item == *tmp->data))
		tmp = tmp->next;

	if (item == *tmp->data){
		curr = tmp;
		return 0;
	}else
		return -1;
}

template <class DType> DType CMGLinkedList<DType>::remove_this()
{
   CMGListNode<DType> * ltmp = curr;
   DType *tmp  = ltmp->data;
   return tmp;
}

template <class DType> DType CMGLinkedList<DType>::remove()
{    
	// Remove CMGListNode after curr, return it's element
	// curr must be valid position in list
	if (curr == NULL)
		return curr->data;

	//CMGListNode* ltmp = curr->next;
	CMGListNode<DType>* ltmp = curr;
	DType tmp  = ltmp->data;

	if( ltmp->next != NULL )
		ltmp->next->prev = curr->prev;
	else                      // Removed tail elem - change tail
		tail = curr->next;

	if( ltmp->prev != NULL )
		ltmp->prev->next = curr->next;
	else                      // Removed head elem - change head
		head = curr->next;

	curr = ltmp->next;
	len--;

	if ((m_releaseData) && (ltmp->data))
  		delete (ltmp->data);
	delete ltmp;
	
	return tmp;
}



template <class DType> int CMGLinkedList<DType>::length() const
{                                     // Return current length of list
  return len;
}


template <class DType> bool CMGLinkedList<DType>::isEmpty() const           // Return TRUE if list is empty
{
  return len == 0;
}

template <class DType> void CMGLinkedList<DType>::toHead()
{
  curr = head;
}

template <class DType> void CMGLinkedList<DType>::toTail()
{
  curr = tail;
}

template <class DType> void CMGLinkedList<DType>::next()
{
  curr = curr->next;
}

template <class DType> void CMGLinkedList<DType>::previous()
{
  curr = curr->prev;
}

template <class DType> bool CMGLinkedList<DType>::eof()
{
	return curr == NULL;
}

template <class DType> bool CMGLinkedList<DType>::bof()
{
	return curr == NULL;
}

template <class DType> DType CMGLinkedList<DType>::current()
{
	return curr->data;
}

#endif