/******************************************************************************
 *
 * Copyright (c) 1999 Palm Computing, Inc. or its subsidiaries.
 * All rights reserved.
 *
 * File: Starter.cpp
 *
 *****************************************************************************/

#include <PalmOS.h>
#include "HTTPDemoRsc.h"
#include "MKHTTPS.h"


//	char buf[4096];


/***********************************************************************
 *
 *   Entry Points
 *
 ***********************************************************************/


/***********************************************************************
 *
 *   Internal Structures
 *
 ***********************************************************************/
typedef struct 
	{
	UInt8 replaceme;
	} StarterPreferenceType;

typedef struct 
	{
	UInt8 replaceme;
	} StarterAppInfoType;

typedef StarterAppInfoType* StarterAppInfoPtr;


/***********************************************************************
 *
 *   Global variables
 *
 ***********************************************************************/
//static Boolean HideSecretRecords;


/***********************************************************************
 *
 *   Internal Constants
 *
 ***********************************************************************/
#define appFileCreator			  'hsDE'
#define appVersionNum              0x01
#define appPrefID                  0x00
#define appPrefVersionNum          0x01

// Define the minimum OS version we support (5.0 for now).
#define ourMinVersion		sysMakeROMVersion(3,0,0,sysROMStageRelease,0)
#define ourMinVersionSecure	sysMakeROMVersion(5,0,0,sysROMStageRelease,0)

/***********************************************************************
 *
 *   Internal Functions
 *
 ***********************************************************************/
static void doGo();
static void setTextToField(unsigned long fldID, char* text);


/***********************************************************************
 *
 * FUNCTION:    RomVersionCompatible
 *
 * DESCRIPTION: This routine checks that a ROM version is meet your
 *              minimum requirement.
 *
 * PARAMETERS:  requiredVersion - minimum rom version required
 *                                (see sysFtrNumROMVersion in SystemMgr.h 
 *                                for format)
 *              launchFlags     - flags that indicate if the application 
 *                                UI is initialized.
 *
 * RETURNED:    error code or zero if rom is compatible
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err RomVersionCompatible(UInt32 requiredVersion, UInt16 launchFlags, bool secure = false)
{
	UInt32 romVersion;

	// See if we're on in minimum required version of the ROM or later.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion)
		{
		if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
			{
			if ( secure )
				FrmAlert(RomIncompatibleSecureAlert);
			else
				FrmAlert(RomIncompatibleAlert);
		
			// Palm OS 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.
			if (romVersion < ourMinVersion)
				{
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
				}
			}
		
		return sysErrRomIncompatible;
		}

	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: This routine returns a pointer to an object in the current
 *              form.
 *
 * PARAMETERS:  formId - id of the form to display
 *
 * RETURNED:    void *
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void * GetObjectPtr(UInt16 objectID)
{
	FormPtr frmP;

	frmP = FrmGetActiveForm();
	return FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID));
}

static void Scrollear()
{
	UInt16 scrollPos;
	UInt16 textHeight;
	UInt16 fieldHeight;
	Int16 maxValue;
	
	FldGetScrollValues((FieldPtr)GetObjectPtr(MainFContentField),
	  				   &scrollPos, &textHeight, &fieldHeight);

	if ( textHeight > fieldHeight )
		maxValue = textHeight - fieldHeight;
	else {
		if ( scrollPos )
			maxValue = scrollPos;
		else
			maxValue = 0;
	}

	SclSetScrollBar((ScrollBarPtr)GetObjectPtr(MainFContectScrollBar),
					scrollPos, 0, maxValue, fieldHeight-1);
}

/***********************************************************************
 *
 * FUNCTION:    MainFormInit
 *
 * DESCRIPTION: This routine initializes the MainForm form.
 *
 * PARAMETERS:  frm - pointer to the MainForm form.
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void MainFormInit(FormPtr /*frmP*/)
{
	Scrollear();
}

void setTextToField(unsigned long fldID, char* text) {
	
	FieldType *fld = (FieldType*)GetObjectPtr(fldID);
	MemHandle memH = FldGetTextHandle(fld);
	char* dstP;

	if (memH) {
		FldSetTextHandle(fld, NULL);
		MemHandleResize(memH, StrLen(text)+1);
	}
	else {
		memH = MemHandleNew(StrLen(text)+1);
	}
	dstP = (char*)MemHandleLock(memH);
	MemMove(dstP, text, StrLen(text)+1);
	
	MemHandleUnlock(memH);
	FldSetTextHandle(fld,memH);
	FldDrawField(fld);
}

void doGo() 
{
	char strNum[13];
	char* mURL;
	char* content;
	unsigned short contentL;
	unsigned short responseCode;
	FieldType *fldP = (FieldType*) GetObjectPtr(MainFURLField);
	
	bool secure = (CtlGetValue((ControlType*)GetObjectPtr(MainSecureCheckbox)) == 0)? false : true;

	if ( RomVersionCompatible((secure)? ourMinVersionSecure : ourMinVersion, sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp, secure) ) 
		return;

	// HTTPConnection
	HTTPConnection* cnn;
	if ( secure )
		cnn = new HTTPSConnection(INPUT_BUFFER_SIZE * 20, OUTPUT_BUFFER_SIZE * 10); //Buffer Size for the connection (Max Page Size), Output buffer
	else
		cnn = new HTTPConnection(INPUT_BUFFER_SIZE * 20, OUTPUT_BUFFER_SIZE * 10); //Buffer Size for the connection (Max Page Size), Output buffer

	// HTTPRequest
	HTTPRequest *req = new HTTPRequest((HTTPConnection*)cnn);
	
	mURL = FldGetTextPtr(fldP);	
	setTextToField(MainFContentField, "");
	setTextToField(MainFContentLengthField, "");
	setTextToField(MainFResponseCodeField, "");
	if (mURL) {
		/* -- Test using .NET Web Service --
		req->domain.set("MyComputer");
		req->path.set("/WebService2/Service1.asmx/ConvertTemperature?dFahrenheit=104");
		*/
		/* -- Test manually setting Headers --
		req->verb = httpGet;
		req->domain.set("www.alerted.com");
		//req->path.set("/alert/visited.asp?usernameEmail=rbudding@alerted.com&event=52");
		req->path.set("/alert/pda/retUser.asp?Username=1");
		req->getHeaders()->remove("Connection");
		req->getHeaders()->remove("Accept");
		req->getHeaders()->remove("User-Agent");
		responseCode = req->execute();
		*/
		/* -- Set Manual --
		req->verb = httpGet;
		req->domain.set("archive.nlm.nih.gov");
		req->port = 8080;
		req->path.set("/mot-see/mdot?citation=12344");
		responseCode = req->execute();
		*/
		responseCode = req->execute(httpGet, mURL, NULL);
		if (req->getLastError()) {
			if (req->getLastError() == errCantResolveHost)
				content = "Cannot Resolve Host";
			else
				content = "Error: URL can't be retrieve.";
			contentL = 0;
		}
		else {
			content = req->getResponse()->getContent();
			//Get content-length header (Not available if transfer-coding: chunked)
			contentL = req->getResponse()->getContentLength();
	        /*
	        MemSet(buf, sizeof(buf), 0);
	        MemMove(buf, content, contentL);
	        */
		}
		setTextToField(MainFContentField, content);
		StrPrintF(strNum, "%u", contentL);
		setTextToField(MainFContentLengthField, strNum);
		StrPrintF(strNum, "%u", responseCode);	
		setTextToField(MainFResponseCodeField, strNum);
	} 
	else {
		setTextToField(MainFContentField, "You must enter a valid address.");
	}
	
	//Release Objects
	delete req;
	delete cnn; 

	Scrollear();
}

static Int16 abs(Int16 i)
{
	return (i < 0)? (i * (-1)) : i;
}

/***********************************************************************
 *
 * FUNCTION:    MainFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "MainForm" of this application.
 *
 * PARAMETERS:  eventP  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventPtr eventP)
{
   Boolean handled = false;
   FormPtr frmP;

	switch (eventP->eType) 
		{
		case frmOpenEvent:
			frmP = FrmGetActiveForm();
			MainFormInit( frmP);
			FrmDrawForm ( frmP);
			setTextToField(MainFURLField, "www.verisign.com");
			handled = true;
			break;

		case ctlSelectEvent:
			switch (eventP->data.ctlSelect.controlID) {
				case MainGoButton:
					doGo();
					break;
			}
			break;
			
		case sclRepeatEvent:
			WinDirectionType direction = (eventP->data.sclRepeat.value < eventP->data.sclRepeat.newValue)? winDown : winUp;
			Int16 linesToScroll = abs(eventP->data.sclRepeat.value - eventP->data.sclRepeat.newValue);
		
			FldScrollField((FieldPtr)GetObjectPtr(MainFContentField), linesToScroll, direction);
			
			// Update the scroll bar
			Scrollear();
			break;
			
		default:
			break;
		}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean AppHandleEvent(EventPtr eventP)
{
	UInt16 formId;
	FormPtr frmP;

	if (eventP->eType == frmLoadEvent)
		{
		// Load the form resource.
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId)
			{
			case MainForm:
				FrmSetEventHandler(frmP, MainFormHandleEvent);
				break;

			default:
//				ErrFatalDisplay("Invalid Form Load Event");
				break;

			}
		return true;
		}
	
	return false;
}


/***********************************************************************
 *
 * FUNCTION:    AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppEventLoop(void)
{
	EventType event;

	do {
		EvtGetEvent(&event, evtWaitForever);

		if (! SysHandleEvent(&event))
			if (! AppHandleEvent(&event))
				FrmDispatchEvent(&event);

	} while (event.eType != appStopEvent);
}


/***********************************************************************
 *
 * FUNCTION:     AppStart
 *
 * DESCRIPTION:  Get the current application's preferences.
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     Err value 0 if nothing went wrong
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err AppStart(void)
{
    StarterPreferenceType prefs;
    UInt16 prefsSize;

	// Read the saved preferences / saved-state information.
	prefsSize = sizeof(StarterPreferenceType);
	if (PrefGetAppPreferences(appFileCreator, appPrefID, &prefs, &prefsSize, true) != 
		noPreferenceFound)
		{
		}
	
   return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppStop(void)
{
   StarterPreferenceType prefs;

	// Write the saved preferences / saved-state information.  This data 
	// will be backed up during a HotSync.
	PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
		&prefs, sizeof (prefs), true);
		
	// Close all the open forms.
	FrmCloseAllForms ();
}


/***********************************************************************
 *
 * FUNCTION:    StarterPalmMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 *
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static UInt32 StarterPalmMain(UInt16 cmd, MemPtr /*cmdPBP*/, UInt16 launchFlags)
{
	Err error;

	error = RomVersionCompatible (ourMinVersion, launchFlags);
	if (error) return (error);

	switch (cmd)
		{
		case sysAppLaunchCmdNormalLaunch:
			error = AppStart();
			if (error) 
				return error;
				
			FrmGotoForm(MainForm);
			AppEventLoop();
			AppStop();
			break;

		default:
			break;

		}
	
	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
UInt32 PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    return StarterPalmMain(cmd, cmdPBP, launchFlags);
}

